import os
import re
import csv
from dataclasses import dataclass
from typing import Dict


def parse_conf(path: str = "./.conf") -> Dict[str, str]:
    """Parses variables from .conf file

    Args:
        path (str): Path to file to parse. Defaults to ./.conf

    Returns:
        Dict[str, str]: Dictionary containing keys and value pairs of vars in file.
    """
    with open(path) as stream:
        contents = stream.read().strip()

    var_declarations = re.findall(r"^[a-zA-Z0-9_]+=.*$", contents, flags=re.MULTILINE)
    reader = csv.reader(var_declarations, delimiter="=")
    bash_vars = dict(reader)

    return bash_vars



def get_part_id() -> str:
    """Get part id from environment variable.
    Returns:
        str: The part id.
    """
    part_id = os.getenv("partId")
    if part_id is None:
        return ""
    return part_id


@dataclass
class Config:
    """Class that holds configuration values for the grader.
    """
    bind_mount: str = "/shared/submission/"
    submission_workdir: str = "./submission/"
    solution_workdir: str = "./solution/"
    solution_file: str = "solution.ipynb"
    solution_file_path: str = os.path.join(solution_workdir, solution_file)
    submission_file: str = "submission.ipynb"
    submission_file_path: str = os.path.join(submission_workdir, submission_file)
    feedback_file_path: str = "/shared/feedback.json"
    latest_version = parse_conf().get("TAG_ID")
    part_id: str = get_part_id()



import ast
import sys
from types import ModuleType
from contextlib import nullcontext
from .io import suppress_stdout_stderr


def compile_module(code_as_str: str, module_name: str, wipe_global_state: bool = False, verbose: bool = True) -> ModuleType:
    """Compiles the string representation of some code and returns a compiled module.
    Args:
        code_as_str (str): Code represented as string.
        module_name (str): Name of the module.
        wipe_global_state (bool): If true then no global state is compiled. Defaults to False.
        verbose (bool): Whether to print out streams as a result of compilation. Defaults to True.
    Returns:
        ModuleType: The actual module that can be used to call functions/variables/etc.
    """
    code_ast = ast.parse(code_as_str)

    if wipe_global_state:
        for node in code_ast.body[:]:
            if not isinstance(node, (ast.FunctionDef, ast.Import, ast.ImportFrom, ast.ClassDef)):
                code_ast.body.remove(node)
        
    with nullcontext() if verbose else suppress_stdout_stderr():
        module = ModuleType(module_name)
        code = compile(code_ast, f"{module_name}.py", "exec")
        sys.modules[module_name] = module
        exec(code, module.__dict__)
        return module


import os
import json
import shutil
import tarfile
import jupytext
from os import devnull
from zipfile import ZipFile
from contextlib import contextmanager, redirect_stderr, redirect_stdout
from .notebook import read_nb, tag_code_cells


@contextmanager
def suppress_stdout_stderr():
    """A context manager that redirects stdout and stderr to devnull"""
    with open(devnull, "w") as fnull:
        with redirect_stderr(fnull) as err, redirect_stdout(fnull) as out:
            yield (err, out)


def tag_notebook(path) -> None:
    """Adds 'graded' tag to all code cells of a notebook.

    Args:
        path (str): Path to the notebook.
    """
    nb = read_nb(path)
    nb = tag_code_cells(nb)
    jupytext.write(nb, path)


def extract_tar(file_path: str, destination: str, post_cleanup: bool = True) -> None:
    """Extracts a tar file unto the desired destination.

    Args:
        file_path (str): Path to tar file.
        destination (str): Path where to save uncompressed files.
        post_cleanup (bool, optional): If true, deletes the compressed tar file. Defaults to True.
    """
    with tarfile.open(file_path, "r") as my_tar:
        my_tar.extractall(destination)

    if post_cleanup and os.path.exists(file_path):
        os.remove(file_path)


def extract_zip(file_path, destination, post_cleanup: bool = True) -> None:
    """Extracts a zip file unto the desired destination.

    Args:
        file_path (str): Path to zip file.
        destination (str): Path where to save uncompressed files.
        post_cleanup (bool, optional): If true, deletes the compressed zip file. Defaults to True.
    """
    with ZipFile(file_path, "r") as zip:
        zip.extractall(destination)

    if post_cleanup and os.path.exists(file_path):
        os.remove(file_path)


def send_feedback(
    score: float,
    msg: str,
    feedback_path: str = "/shared/feedback.json",
    err: bool = False,
) -> None:
    """Sends feedback to the learner.
    Args:
        score (float): Grading score to show on Coursera for the assignment.
        msg (str): Message providing additional feedback.
        feedback_path (str): Path where the json feedback will be saved. Defaults to /shared/feedback.json
        err (bool, optional): True if there was an error while grading. Defaults to False.
    """

    post = {"fractionalScore": score, "feedback": msg}
    print(json.dumps(post))

    with open(feedback_path, "w") as outfile:
        json.dump(post, outfile)

    if err:
        exit(1)

    exit(0)


def copy_submission_to_workdir(
    dir_origin: str = "/shared/submission/",
    dir_destination: str = "./submission/",
    file_name: str = "submission.ipynb",
) -> None:
    """Copies submission file from bind mount directory into working directory.
    Args:
        dir_origin (str): Origin directory.
        dir_destination (str): Target directory.
        file_name (str): Name of the file.
    """

    file_initial_path = os.path.join(dir_origin, file_name)
    file_final_path = os.path.join(dir_destination, file_name)

    shutil.copyfile(file_initial_path, file_final_path)



import os
from .templates import load_templates

def write_file(filename, template):
    with open(filename, 'w') as f:
        f.write(template)


def init():
    
    dockerfile, makefile, conf = load_templates()
    write_file("./Dockerfile", dockerfile)
    write_file("./Makefile", makefile)
    write_file("./.conf", conf)
    write_file("./requirements.txt", "dlai-grader")
    write_file("./grader.py", "")
    write_file("./entry.py", "")
    os.makedirs("mount")
    os.makedirs("solution")
    os.makedirs("submission")



from . import compiler
from . import config
from . import notebook
from . import io
from . import grading
from . import generator

__version__ = "0.2.1"
__author__ = 'Andres Zarta'
__credits__ = 'DeepLearning.AI'



def load_templates():
    specialization = input("Name of the specialization: ")
    course = input("Number of the course: ")
    week = input("Number of the week: ")

    dockerfile = """
FROM frolvlad/alpine-miniconda3:python3.7

RUN apk update && apk add libstdc++

COPY requirements.txt .

RUN pip install -r requirements.txt && \
rm requirements.txt

RUN mkdir /grader && \ 
mkdir /grader/submission

COPY data/ /grader/data/
COPY entry.py /grader/entry.py
COPY grader.py /grader/grader.py

RUN chmod a+rwx /grader/

WORKDIR /grader/

ENTRYPOINT ["python", "entry.py"]
    """[
        1:
    ]

    conf = f"""
ASSIGNMENT_NAME=C{course}W{week}_Assignment
IMAGE_NAME={specialization}c{course}w{week}-grader
TAG_ID=1
SUB_DIR=mount
MEMORY_LIMIT=4096
    """[
        1:
    ]

    makefile = """
include .conf

OS := $(shell uname)

learner:
	[ "$(ls -A ./$(SUB_DIR))" ] && rm ./$(SUB_DIR)/* && echo "Deleted previous version"|| echo "Empty"
	cp mount/submission.ipynb learner/$(ASSIGNMENT_NAME)_Solution.ipynb
	learner -fn learner/$(ASSIGNMENT_NAME)_Solution.ipynb 
	rm learner/$(ASSIGNMENT_NAME)_Solution.ipynb && rm learner/$(ASSIGNMENT_NAME)_Solution.py && rm learner/$(ASSIGNMENT_NAME).py

build:
	docker build -t $(IMAGE_NAME):$(TAG_ID) .

entry:
	docker run -it --rm --mount type=bind,source=$(PWD)/mount,target=/shared/submission --mount type=bind,source=$(PWD),target=/grader/ --entrypoint ash $(IMAGE_NAME):$(TAG_ID)

test:
	docker run -it --rm --mount type=bind,source=$(PWD)/mount,target=/shared/submission --mount type=bind,source=$(PWD),target=/grader/ --entrypoint pytest $(IMAGE_NAME):$(TAG_ID)

grade:
	gradethis $(PARTIDS) $(IMAGE_NAME):$(TAG_ID) $(SUB_DIR) $(MEMORY_LIMIT)

mem:
	memthis $(PARTIDS)

zip:
	zip -r $(IMAGE_NAME)$(TAG_ID).zip .

clean:
	find . -maxdepth 1 -type f -name "*.zip" -exec rm {} +
	docker rmi $(docker images --filter "dangling=true" -q --no-trunc)
	docker rm $(docker ps -qa --no-trunc --filter "status=exited")

upload:
	coursera_autograder --timeout 1800 upload --grader-memory-limit $(MEMORY_LIMIT) --grading-timeout 1800 $(IMAGE_NAME)$(TAG_ID).zip $(COURSE_ID) $(ITEM_ID) $(PART_ID)

move-zip:
	if [[ "$(OS)" == "Darwin" ]];    \
	then    \
		mv $(IMAGE_NAME)$(TAG_ID).zip /Users/andreszarta/Desktop/upload-temp;    \
	else    \
		mv $(IMAGE_NAME)$(TAG_ID).zip $$dlai/upload;    \
	fi

move-learner:
	if [[ "$(OS)" == "Darwin" ]];    \
	then    \
		cp learner/$(ASSIGNMENT_NAME).ipynb /Users/andreszarta/Desktop/upload-temp;    \
	else    \
		cp learner/$(ASSIGNMENT_NAME).ipynb $$dlai/upload;    \
	fi
    """[
        1:
    ]

    return dockerfile, makefile, conf
